package com.landray.kmss.sys.attachment.restservice.foxit.controller;

import java.io.OutputStream;
import java.io.InputStream;
import org.apache.commons.io.IOUtils;
import com.landray.kmss.sys.attachment.io.IOUtil;
import com.landray.kmss.sys.filestore.location.util.SysFileLocationUtil;
import java.io.File;
import java.util.Iterator;
import com.landray.kmss.sys.attachment.restservice.foxit.controller.dao.FileInfoMinioBase;
import com.landray.kmss.sys.filestore.scheduler.third.foxit.dto.FoxitObserverParameter;
import com.landray.kmss.sys.attachment.restservice.foxit.controller.dao.Results;
import org.springframework.web.bind.annotation.RequestBody;
import com.landray.kmss.sys.attachment.restservice.foxit.controller.dao.CommonCallbackParam;
import com.alibaba.fastjson.JSONObject;
import java.util.HashMap;
import java.util.Map;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RequestMethod;
import com.landray.kmss.sys.filestore.model.SysAttFile;
import java.util.List;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import com.landray.kmss.util.StringUtil;
import org.springframework.web.bind.annotation.RequestParam;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;
import com.landray.kmss.util.SpringBeanUtil;
import java.util.Observer;
import org.slf4j.LoggerFactory;
import com.landray.kmss.sys.filestore.service.ISysAttUploadService;
import com.landray.kmss.sys.filestore.convert.service.CallbackHandle;
import com.landray.kmss.sys.filestore.service.ISysFileConvertQueueService;
import com.landray.kmss.sys.attachment.service.ISysAttMainCoreInnerService;
import com.landray.kmss.sys.attachment.integrate.foxit.ISysAttachmentFoxitProvider;
import com.landray.kmss.sys.filestore.convert.observer.ConvertObserver;
import com.landray.kmss.sys.filestore.convert.observer.ConvertObservable;
import org.slf4j.Logger;
import com.landray.kmss.web.annotation.RestApi;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.landray.kmss.common.rest.controller.BaseController;

@RestController
@RequestMapping({ "/api/sys-attachment/foxit" })
@RestApi(docUrl = "/sys/attachment/restservice/sysAttFoxitRestServiceHelp.jsp", name = "SysAttFoxitController", resourceKey = "sys-attachment:attachment.foxit.controller.title")
public class SysAttFoxitController extends BaseController
{
    private static final Logger logger;
    private static final String FOXIT_DOC_CONTENT_TYPE_PDF = "application/pdf";
    private static final String FOXIT_DOC_CONTENT_TYPE_OFD = "application/ofd";
    private static final String FOXIT_CONVERT_DOC_OFD = "toOFD-Foxit_ofd";
    private static final String FOXIT_CONVERT_DOC_PDF = "toPDF-Foxit_pdf";
    private static final String FOXIT_CONVERT_FILE_PATH = "_convert";
    private static final String FOXIT_CONVERT_FILE_PDF = "_pdf";
    private static final String FOXIT_CONVERT_FILE_OFD = "_ofd";
    private static final String FOXIT_CONVERT_TO_PDF = "toPDF";
    private static final String FOXIT_CONVERT_TO_OFD = "toOFD";
    private static final String FOXIT_DOC_SUFFIX_OFD = ".ofd";
    private static final String FOXIT_DOC_SUFFIX_PDF = ".pdf";
    private static final String PERMISSION_OFF = "0";
    private static final String PERMISSION_ON = "1";
    private static final String PERMISSION_TWO = "2";
    private static ConvertObservable convertObservable;
    private static ConvertObserver convertObserver;
    private ISysAttachmentFoxitProvider sysAttachmentFoxitProvider;
    private ISysAttMainCoreInnerService sysAttMainService;
    private ISysFileConvertQueueService convertQueueService;
    private static CallbackHandle callbackHandle;
    private ISysAttUploadService sysAttUploadService;
    
    static {
        logger = LoggerFactory.getLogger(SysAttFoxitController.class);
        SysAttFoxitController.convertObservable = new ConvertObservable();
        SysAttFoxitController.convertObserver = new ConvertObserver(getCallbackHandle());
        SysAttFoxitController.convertObservable.addObserver((Observer)SysAttFoxitController.convertObserver);
        SysAttFoxitController.callbackHandle = null;
    }
    
    public SysAttFoxitController() {
        this.sysAttachmentFoxitProvider = null;
        this.sysAttUploadService = null;
    }
    
    private ISysAttachmentFoxitProvider getSysAttachmentFoxitProvider() {
        if (this.sysAttachmentFoxitProvider == null) {
            this.sysAttachmentFoxitProvider = (ISysAttachmentFoxitProvider)SpringBeanUtil.getBean("foxitProvider");
        }
        return this.sysAttachmentFoxitProvider;
    }
    
    private ISysAttMainCoreInnerService getSysAttMainService() {
        if (this.sysAttMainService == null) {
            this.sysAttMainService = (ISysAttMainCoreInnerService)SpringBeanUtil.getBean("sysAttMainService");
        }
        return this.sysAttMainService;
    }
    
    private ISysFileConvertQueueService getConvertQueueService() {
        if (this.convertQueueService == null) {
            this.convertQueueService = (ISysFileConvertQueueService)SpringBeanUtil.getBean("sysFileConvertQueueService");
        }
        return this.convertQueueService;
    }
    
    private static CallbackHandle getCallbackHandle() {
        if (SysAttFoxitController.callbackHandle == null) {
            SysAttFoxitController.callbackHandle = (CallbackHandle)SpringBeanUtil.getBean("foxitCallbackHandleImpl");
        }
        return SysAttFoxitController.callbackHandle;
    }
    
    private ISysAttUploadService getAttUploadService() {
        if (this.sysAttUploadService == null) {
            this.sysAttUploadService = (ISysAttUploadService)SpringBeanUtil.getBean("sysAttUploadService");
        }
        return this.sysAttUploadService;
    }
    
    @RequestMapping(value = { "/document" }, method = { RequestMethod.GET })
    @ResponseBody
    public void download(final HttpServletRequest request, final HttpServletResponse response, @RequestParam final String fileId, @RequestParam final String sysToken) throws Exception {
        final String requestToken = sysToken;
        if (!this.getSysAttachmentFoxitProvider().checkToken(requestToken)) {
            SysAttFoxitController.logger.error("福昕阅读请求系统下载文件，校验token失败:{}", requestToken);
            return;
        }
        try {
            final String fdId = fileId;
            if (StringUtil.isNull(fdId)) {
                SysAttFoxitController.logger.error("请求下载文件fdId为空");
                return;
            }
            final String[] l_fdId = fdId.split(";");
            final List<SysAttMain> sysAttMains = this.getSysAttMainService().findModelsByIds(l_fdId);
            if (sysAttMains == null || sysAttMains.size() <= 0) {
                SysAttFoxitController.logger.error("请求下载的文件不存在，fdId:{}", fdId);
                return;
            }
            final SysAttMain sysAttMain = sysAttMains.get(0);
            final String contentType = sysAttMain.getFdContentType();
            final String fileName = sysAttMain.getFdFileName();
            final SysAttFile sysAttFile = ((ISysAttMainCoreInnerService)SpringBeanUtil.getBean("sysAttMainService")).getFile(sysAttMain.getFdId());
            final String filePath = sysAttFile.getFdFilePath();
            if (SysAttFoxitController.logger.isDebugEnabled()) {
                SysAttFoxitController.logger.debug("福昕阅读下载EKP文件路径:{},文件名:{}", filePath, fileName);
            }
            if (fileName.toLowerCase().endsWith(".ofd") || fileName.toLowerCase().endsWith(".pdf")) {
                this.outputFile(response, sysAttMain.getFdFileName(), contentType, filePath);
                return;
            }
            this.outputConvertFile(response, filePath, fileId);
        }
        catch (final Exception e) {
            SysAttFoxitController.logger.error("Foxit下载EKP的文件出现异常:" + e.getStackTrace());
        }
    }
    
    @ResponseBody
    @RequestMapping(value = { "/document/permission" }, method = { RequestMethod.GET })
    public Map getPermission(final HttpServletRequest request, @RequestParam final String userId, @RequestParam final String fileId, @RequestParam final String canPrint) throws Exception {
        if (SysAttFoxitController.logger.isDebugEnabled()) {
            SysAttFoxitController.logger.debug("Foxit调用权限接口,userId:{},fileId:{},canPrint:{}", userId, fileId, canPrint);
        }
        final Map<String, Object> resultMap = new HashMap<String, Object>();
        if (StringUtil.isNull(fileId)) {
            SysAttFoxitController.logger.error("Foxit调用权限接口，文件ID为空!导致无法获取部分控件权限.");
        }
        resultMap.put("ret", "0");
        resultMap.put("message", "success");
        final Map<String, Object> data = this.getPermissionData(canPrint);
        resultMap.put("data", data);
        if (SysAttFoxitController.logger.isDebugEnabled()) {
            SysAttFoxitController.logger.debug("Foxit获取权限参数:{}", JSONObject.toJSONString((Object)resultMap));
        }
        return resultMap;
    }
    
    @ResponseBody
    @RequestMapping(value = { "/document/getExplicitWatermarkInfos" }, method = { RequestMethod.GET })
    public Map getExplicitWatermarkInfos(final HttpServletRequest request, @RequestParam final String userId, @RequestParam final String fileId) throws Exception {
        final Map<String, Object> resultMap = new HashMap<String, Object>();
        resultMap.put("ret", "0");
        resultMap.put("message", "success");
        final Map<String, Object> data = new FoxitWaterInfo().getBaseInfo().getWordWater(request, userId).getPicWater(request).getData();
        resultMap.put("data", data);
        if (SysAttFoxitController.logger.isDebugEnabled()) {
            SysAttFoxitController.logger.debug("Foxit获取水印信息:{}", JSONObject.toJSONString((Object)resultMap));
        }
        return resultMap;
    }
    
    @ResponseBody
    @RequestMapping(value = { "/convert/callback" }, method = { RequestMethod.POST })
    public Results convertCallback(@RequestBody final CommonCallbackParam req, final HttpServletRequest request) {
        if (SysAttFoxitController.logger.isDebugEnabled()) {
            SysAttFoxitController.logger.debug("福昕转换回调信息是:{}", JSONObject.toJSONString((Object)req));
        }
        final List<FileInfoMinioBase> files = req.getFiles();
        final FoxitObserverParameter foxitObserverParameter = new FoxitObserverParameter();
        for (final FileInfoMinioBase file : files) {
            foxitObserverParameter.setFileId(file.getFileId());
            foxitObserverParameter.setDownloadUrl(file.getDownloadPath());
        }
        foxitObserverParameter.setRetCode(req.getCode());
        SysAttFoxitController.convertObservable.handleConvertCallback((Object)foxitObserverParameter);
        return Results.ok((Object)req);
    }
    
    public Boolean outputConvertFile(final HttpServletResponse response, final String filePath, final String fileId) throws Exception {
        final String fPath = String.valueOf(filePath) + "_convert";
        if (SysAttFoxitController.logger.isDebugEnabled()) {
            SysAttFoxitController.logger.debug("Foxit下载文件根路径:{}", fPath);
        }
        final String ofdPath = String.valueOf(fPath) + File.separator + "toOFD-Foxit_ofd";
        final SysAttFile sysAttFile = ((ISysAttMainCoreInnerService)SpringBeanUtil.getBean("sysAttMainService")).getFile(fileId);
        if (sysAttFile == null) {
            SysAttFoxitController.logger.warn("文件正在转换中！！！");
            return false;
        }
        final String FilePath = this.getAttUploadService().getAbsouluteFilePath(sysAttFile.getFdId());
        final String bakf = String.valueOf(FilePath) + "_bak";
        final SysAttFile file = this.getAttUploadService().getFileById(sysAttFile.getFdId());
        final String pathPrefix = (file.getFdCata() == null) ? null : file.getFdCata().getFdPath();
        final Boolean exist = SysFileLocationUtil.getProxyService(file.getFdAttLocation()).doesFileExist(bakf, pathPrefix);
        if (exist) {
            this.outputFile(response, "toOFD-Foxit_ofd", "application/ofd", ofdPath);
            return true;
        }
        final String pdfPath = String.valueOf(fPath) + File.separator + "toPDF-Foxit_pdf";
        if (exist) {
            this.outputFile(response, "toPDF-Foxit_pdf", "application/pdf", pdfPath);
            return true;
        }
        String fName = "";
        String cType = "";
        boolean flag = false;
        final List<String> fileNames = SysFileLocationUtil.getProxyService().listFileNames(fPath);
        for (final String fileName : fileNames) {
            if (fileName.endsWith("_pdf")) {
                final int last = fileName.lastIndexOf("/");
                fName = fileName.substring(last + 1, fileName.length());
                cType = "application/pdf";
                flag = true;
                break;
            }
            if (fileName.endsWith("_ofd")) {
                final int last = fileName.lastIndexOf("/");
                fName = fileName.substring(last + 1, fileName.length());
                cType = "application/ofd";
                flag = true;
                break;
            }
        }
        String name = "";
        if (fName.contains("toPDF")) {
            name = String.valueOf(fName) + ".pdf";
        }
        else if (fName.contains("toOFD")) {
            name = String.valueOf(fName) + ".ofd";
        }
        if (!flag) {
            SysAttFoxitController.logger.warn("文件正在转换中！！！");
            return false;
        }
        this.outputFile(response, name, cType, String.valueOf(fPath) + File.separator + fName);
        return true;
    }
    
    public void outputFile(final HttpServletResponse response, final String filename, final String contentType, final String filePath) throws Exception {
        InputStream in = null;
        OutputStream out = null;
        try {
            response.setContentType(contentType);
            response.setHeader("Content-Disposition", "attachment;filename=\"" + new String(filename.getBytes("utf-8"), "ISO8859-1") + "\"");
            if (SysAttFoxitController.logger.isDebugEnabled()) {
                SysAttFoxitController.logger.debug("请求下载文件，EKP文件所处位置:{}", filePath);
            }
            in = SysFileLocationUtil.getProxyService().readFile(filePath);
            if (in == null) {
                SysAttFoxitController.logger.error("文件流不存在，请求查检文件路径:{}", filePath);
            }
            out = (OutputStream)response.getOutputStream();
            IOUtil.write(in, out);
        }
        catch (final Exception e) {
            if (out != null) {
                IOUtils.closeQuietly(out);
            }
            if (in != null) {
                IOUtils.closeQuietly(in);
            }
            SysAttFoxitController.logger.error("Foxit下载EKP的文件出现异常:" + e.getStackTrace());
            return;
        }
        finally {
            if (out != null) {
                IOUtils.closeQuietly(out);
            }
            if (in != null) {
                IOUtils.closeQuietly(in);
            }
        }
        if (out != null) {
            IOUtils.closeQuietly(out);
        }
        if (in != null) {
            IOUtils.closeQuietly(in);
        }
    }
    
    public Map<String, Object> getPermissionData(final String canPrint) {
        return new FoxitPermission().setPermission("2").setHead("0").setOpenFileBtn("0").setSaveBtn("0").setPrintBtn(canPrint).setGoToPageBox("1").setZoomPageBox("1").setPageLayoutBtn("0").setHandToolBtn("0").setTextSelectBtn("0").setHeightLightBtn("0").setUnderlineBtn("0").setPencilBtn("0").setDrawingAnnotBtn("0").setCommentsBtn("0").setElecSignatureBtn("0").setCheckElecSignatureBtn("0").setRotateSwitchBtn("0").setExportBtn("0").setOutlineBtn("0").setThumbnailBtn("1").setCommentListBtn("0").setSearchBtn("1").setSemanticTreeBtn("0").setAttachmentBtn("0").setPageRange("ALL").getData();
    }
}
